/*
     This small program reads a process ID off of the command line and
     then calls the Win32 function SetProcessWorkingSetSize to trim the
     working set for the process to zero pages. The process will then
     have to use page faults to build its working set back up. You can
     use the Task Manager to observe the working set suddenly go to zero
     and then see the page faults and the growth of the working set.

     You can also try setting the minWorkingSetSize and maxWorkingSetSize
     to something small (just a few pages) and see what happens to the process.
*/
#include <windows.h>
#include <stdio.h>

// prototype for function that prints meaningful error messages
void printError(char* functionName);

int main(int argc, char *argv[])
{
   HANDLE processHandle; // handle for process to be trimmed
   int processID;        // pid of process to be trimmed

   DWORD minWorkingSetSize = 0xffffffff;  // this is -1 in 2s complement form
   DWORD maxWorkingSetSize = 0xffffffff;

   // get the process ID from the command line
   if (argc < 2)
   {
      printf("usage: trimPages <pid>");
      return -1;
   }

   processID = atoi(argv[1]);
   processHandle = OpenProcess(PROCESS_ALL_ACCESS, FALSE, processID);
   if (processHandle == NULL)
   {  printError("OpenProcess");
   }

   // use the process handle to trim the process working set to zero pages
   if( !SetProcessWorkingSetSize(processHandle, minWorkingSetSize, maxWorkingSetSize) )
   {  printError("SetProcessWorkingSetSize");
   }

   // report the min and max working set size for the process that was just trimed
   if( !GetProcessWorkingSetSize(processHandle, &minWorkingSetSize, &maxWorkingSetSize) )
   {  printError("GetProcessWorkingSetSize");
   }
   printf("pid = %d, min = %d, max = %d", processID,
                                          (int)minWorkingSetSize,
                                          (int)maxWorkingSetSize);

   return 0;
}//main


/*************************************************************************************
    This method is used to print out useful error messages
    for Win32 (but not C Library) function calls.
*/
void printError(char* functionName)
{
   LPVOID lpMsgBuf;
   int error_no;
   error_no = GetLastError();
   FormatMessage(
         FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
         NULL,
         error_no,
         MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
         (LPTSTR) &lpMsgBuf,
         0,
         NULL
   );
   // Display the string.
   fprintf(stderr, "\n%s failed on error %d: ", functionName, error_no);
   fprintf(stderr, (LPTSTR)lpMsgBuf);
   // Free the buffer.
   LocalFree( lpMsgBuf );
   ExitProcess(1);  // terminate the program
}//printError
